import Foundation
import Apollo
import GraphAPI
import ApolloCodegenLib

@main
public struct DeferClient {
  public static func main() async {
    //await generateCode() <-- Uncomment this to generate the deferred operation model

    fetchExampleQuery()
  }

  static func generateCode() async {
    CodegenLogger.level = .warning

    let config = ApolloCodegenConfiguration(
      schemaNamespace: "GraphAPI",
      input: .init(schemaPath: "**/*.graphqls", operationSearchPaths: ["**/*.graphql"]),
      output: .init(
        schemaTypes: .init(path: "./GraphAPI", moduleType: .swiftPackageManager),
        operations: .inSchemaModule
      )
    )

    // TODO: You must provide the path to your project root in the `fileURLWithPath` property below.
    try! await ApolloCodegen.build(with: config, withRootURL: URL(fileURLWithPath: "<your path here>"))
  }

  static func fetchExampleQuery() {
    class MySessionClient: URLSessionClient { }

    let url = URL(string: "http://localhost:4000/")!
    let store = ApolloStore()
    let interceptorProvider = DefaultInterceptorProvider(client: MySessionClient(), store: store)
    let networkTransport = RequestChainNetworkTransport(interceptorProvider: interceptorProvider, endpointURL: url)
    let client = ApolloClient(networkTransport: networkTransport, store: store)

    print("Connecting to \(url.absoluteString)")

    DispatchQueue.main.async {
      client.fetch(query: ExampleQuery()) { result in
        switch (result) {
        case let .success(data):
          guard let allProducts = data.data?.allProducts else {
            print("Query Success but cannot access data!")
            return
          }

          print("""
          Received:
          - count: \(allProducts.count)
          """)

          for (index, product) in allProducts.enumerated() {
            print("""
            - \(index) id: \(String(describing: product?.id))
            - \(index) sku: \(String(describing: product?.sku))
            - \(index) dimensions.size: \(String(describing: product?.asProduct?.fragments.additional?.dimensions?.size))
            - \(index) variation.id: \(String(describing: product?.asProduct?.fragments.additional?.variation?.id))
            - \(index) variation.name: \(String(describing: product?.asProduct?.fragments.additional?.variation?.name))

            - \(String(describing: product?.asProduct?.fragments.$additional))

            """)
          }

        case let .failure(error):
          print("Query Failure! \(error)")
        }
      }
    }

    dispatchMain()
  }
}
